
#include "vtkCPVTKPipeline.h"
#include <vtkLiveInsituLink.h>
#include <vtkCPProcessor.h>

#include <vtkNew.h>
#include <vtkSMProxyManager.h>
#include <vtkCPDataDescription.h>
#include <vtkCPInputDataDescription.h>
#include <vtkImageData.h>
#include <vtkPointData.h>
#include <vtkDoubleArray.h>

void Init() {

    printf("In Init Catalyst \n");

    if(Processor == NULL) {
        Processor = vtkCPProcessor::New();
        Processor->Initialize();
        vtkNew<vtkCPVTKPipeline> pipeline;
        pipeline->Initialize();
        Processor->AddPipeline(pipeline.GetPointer());
    }

    if(LiveLink == NULL) {
        LiveLink = vtkLiveInsituLink::New();
        LiveLink->SetInsituPort(22222);
        LiveLink->SetHostname("localhost");
        LiveLink->SetProcessType(vtkLiveInsituLink::INSITU);
        vtkSMProxyManager* proxyManager = vtkSMProxyManager::GetProxyManager();
        vtkSMSessionProxyManager* sessionProxyManager =  proxyManager->GetActiveSessionProxyManager();
        LiveLink->InsituInitialize(sessionProxyManager);
    }
}
//
/**************************************************************************/
void CatalystCoProcess(int timeStep, double time, unsigned int numPoints[3], double spacing[3], double* field) {

    printf("%i - %f ",timeStep,time);
//1. Specify current timeStep and time for Catalyst
//2. Check with Catalyst if anything needs to be done this call
//3. If nothing needs to be done this call, return
//4. Create VTK grid
//5. Create VTK fields and associate with VTK grid
//6. Specify VTK grid for Catalyst
//7. Call Catalyst to perform co-processing (i.e. execute VTK pipelines)


    vtkCPDataDescription* dataDescription = vtkCPDataDescription::New();
    dataDescription->SetTimeData(time, timeStep);
    dataDescription->AddInput("input");


    if(Processor->RequestDataDescription(dataDescription) != 0) {
        printf(" generating the vtkImageData ");
        
        vtkImageData* grid = vtkImageData::New();
        grid->SetOrigin(0,0,0);
        grid->SetSpacing(1,1,1);
        grid->SetDimensions(numPoints[0],numPoints[1],numPoints[2]);
        grid->SetExtent(0, numPoints[0] -1, 0, numPoints[1] - 1, 0, numPoints[2]- 1);
        dataDescription->GetInputDescriptionByName("input")->SetGrid(grid);
        
        // Create a field associated with points
        vtkDoubleArray* array = vtkDoubleArray::New();
        array->SetName("temperature");
        array->SetArray(field, grid->GetNumberOfPoints(), 1);
        grid->GetPointData()->AddArray(array);
        grid->GetPointData()->SetScalars(array);
        
        grid->Delete();
        array->Delete();

        Processor->CoProcess(dataDescription);
    } else {
        printf(" non update needed ");
    }
    printf(" \n ");
    
    //this is for the live visu translated from python 
    //stay in the loop while the simulation is paused
    while(true) {
        // Update the simulation state, extracts and simulationPaused
        // from ParaView Live
        LiveLink->InsituUpdate(time,timeStep );
        
        // why in the python they do an update to the pipelines, who can I do the same thing in c++??

        LiveLink->InsituPostProcess(time, timeStep);

        if (LiveLink->GetSimulationPaused()) {
            if (LiveLink->WaitForLiveChange()) {
                break;
            }
        } else {
            break;
        }

    }

}
//-------------------------------------------------------------------------
/* Function to close all still open tcpip connections */
int CloseAll() {

    if(LiveLink) {
        LiveLink->DropLiveInsituConnection();
        LiveLink->Delete();
    }
    if(Processor) {
        Processor->Delete();
        Processor = NULL;
    }
    return 0;
}
//----------------------------------------------------------------------------

