#include "vtkCPVTKPipeline.h"

#include <vtkCommunicator.h>
#include <vtkCPDataDescription.h>
#include <vtkDataArray.h>
#include <vtkCPInputDataDescription.h>
#include <vtkMultiProcessController.h>
#include <vtkNew.h>
#include <vtkObjectFactory.h>
#include <vtkPointData.h>
#include <vtkPVArrayCalculator.h>
#include <vtkPVTrivialProducer.h>
#include <vtkSMProxyManager.h>
#include <vtkImageData.h>
#include <vtkDoubleArray.h>
#include <vtkProcessModule.h>
#include <vtkSMProxy.h>
#include <vtkSMSessionProxyManager.h>
#include <string>
#include <vtksys/ios/sstream>



vtkCPProcessor* Processor = NULL;
vtkLiveInsituLink* LiveLink = NULL;

vtkStandardNewMacro(vtkCPVTKPipeline);

//----------------------------------------------------------------------------
vtkCPVTKPipeline::vtkCPVTKPipeline(){
  this->OutputFrequency = 0;
  this->input = vtkPVTrivialProducer::New();
}
//----------------------------------------------------------------------------
vtkCPVTKPipeline::~vtkCPVTKPipeline(){ 
  this->input->Delete();
  this->input =0;
}
//----------------------------------------------------------------------------
void vtkCPVTKPipeline::Initialize(){
  this->OutputFrequency = 10;
  }
//----------------------------------------------------------------------------
int vtkCPVTKPipeline::RequestDataDescription(vtkCPDataDescription* dataDescription)
{
  if(!dataDescription)
    {
    vtkWarningMacro("dataDescription is NULL.");
    return 0;
    }

  if( dataDescription->GetForceOutput() == true || (this->OutputFrequency != 0 && dataDescription->GetTimeStep() % this->OutputFrequency == 0) )
    {
    dataDescription->GetInputDescriptionByName("input")->AllFieldsOn();
    dataDescription->GetInputDescriptionByName("input")->GenerateMeshOn();
    return 1;
    }
  return 0;
}

//----------------------------------------------------------------------------
int vtkCPVTKPipeline::CoProcess(vtkCPDataDescription* dataDescription){
  if(!dataDescription){
    vtkWarningMacro("DataDescription is NULL");
    return 0;
  }
  
  vtkCPInputDataDescription* inputdescription = dataDescription->GetInputDescriptionByName("input");

  vtkImageData* grid = vtkImageData::SafeDownCast(inputdescription->GetGrid());
  if(grid == NULL) {
    vtkWarningMacro("DataDescription is missing input unstructured grid.");
    return 0;
  }
  
  if(this->RequestDataDescription(dataDescription) == 0){
    return 1;
  }
  
  printf("working ");
  if( !inputdescription->GetIfGridIsNecessary() ) return 1;
  
  printf("*");
  
  this->input->SetOutput(grid);
  int ext[6] ;
  inputdescription->GetWholeExtent(ext);
  this->input->SetWholeExtent (ext);
  this->input->Update();
  

  return 1;
}

//----------------------------------------------------------------------------
void vtkCPVTKPipeline::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os, indent);
  os << indent << "OutputFrequency: " << this->OutputFrequency << "\n";
}
